@@ -55,7 +55,7 @@ def different_ods(ods1, ods2, ignore_type=False, ignore_empty=False, ignore_keys
         keys_to_ignore.extend(default_keys_to_ignore)
 
     def is_ignored(k):
-        return any([o2u(k).endswith(end) for end in keys_to_ignore])
+        return any(o2u(k).endswith(end) for end in keys_to_ignore)
 
     k1 = set(ods1.keys())
     k2 = set(ods2.keys())
@@ -77,7 +77,7 @@ def different_ods(ods1, ods2, ignore_type=False, ignore_empty=False, ignore_keys
                     differences.append(f'DIFF: `{k}` differ in value')
             elif not ignore_type and type(ods1[k]) != type(ods2[k]):
                 differences.append(f'DIFF: `{f}` differ in type: {type(ods1[k])} vs type(ods2[k])')
-            elif numpy.atleast_1d(is_uncertain(ods1[k])).any() or numpy.atleast_1d(is_uncertain(ods2[k])).any():
+            elif is_uncertain(ods1[k]) or is_uncertain(ods2[k]):
                 v1 = nominal_values(ods1[k])
                 v2 = nominal_values(ods2[k])
                 d1 = std_devs(ods1[k])
@@ -216,8 +216,7 @@ def is_uncertain(var):
     """
     :param var: Variable or array to test
 
-    :return: True if variable is instance of uncertainties or
-             array of shape var with elements indicating uncertainty
+    :return: True if input variable or array is uncertain
     """
 
     def _uncertain_check(x):
@@ -225,10 +224,13 @@ def is_uncertain(var):
 
     if isinstance(var, str):
         return False
-    elif numpy.iterable(var):
-        tmp = numpy.array(var).flat
-        tmp = numpy.array(list(map(_uncertain_check, tmp)))
-        return numpy.reshape(tmp, numpy.array(var).shape)
+    elif numpy.iterable(var) or isinstance(var, numpy.ndarray):  # isinstance needed for 0D arrays from squeeze
+        tmp = numpy.array(var)
+        if tmp.dtype not in ['O', 'object']:
+            return False
+        else:
+            # the argument of any is a generator object (using a list slows things down)
+            return any(_uncertain_check(x) for x in tmp.flat)
     else:
         return _uncertain_check(var)
 
@@ -307,9 +309,8 @@ def json_dumper(obj, objects_encode=True):
             return obj.toJSON()
 
     else:
-        tmp = is_uncertain(obj)
-        if numpy.any(numpy.atleast_1d(tmp)):
-            if not len(numpy.array(tmp).shape):
+        if is_uncertain(obj):
+            if not len(numpy.array(obj).shape):
                 return dict(__ufloat__=nominal_values(obj), __ufloat_std__=std_devs(obj))
             else:
                 nomv = nominal_values(obj)
@@ -853,88 +854,15 @@ def omas_global_quantities(imas_version=omas_rcparams['default_imas_version']):
     return _global_quantities[imas_version]
 
 
-_p2l_cache = {}
+try:
+    import pyximport
 
-
-def p2l(key):
-    """
-    Converts the many different ways of addressing an ODS path to a list of keys ['bla',0,'bla']
-
-    :param key: ods location in some format
-
-    :return: list of keys that make the ods path
-    """
-    if isinstance(key, list):
-        return key
-
-    if isinstance(key, tuple):
-        return list(key)
-
-    if isinstance(key, (int, numpy.integer)):
-        return [int(key)]
-
-    if isinstance(key, str) and not ('.' in key or '[' in key):
-        if len(key):
-            return [key]
-        else:
-            return []
-
-    if key is None:
-        raise TypeError('OMAS key cannot be None')
-
-    if isinstance(key, dict):
-        raise TypeError('OMAS key cannot be of type dictionary')
-
-    key0 = ''.join(key)
-    if key0 in _p2l_cache:
-        return copy.deepcopy(_p2l_cache[key0])
-
-    if not isinstance(key, (list, tuple)):
-        key = str(key).replace('[', '.').replace(']', '').split('.')
-
-    key = [k for k in key if k]
-    for k, item in enumerate(key):
-        try:
-            key[k] = int(item)
-        except ValueError:
-            pass
-
-    if len(_p2l_cache) > 1000:
-        _p2l_cache.clear()
-    _p2l_cache[key0] = copy.deepcopy(key)
-
-    return key
-
-
-def l2i(path):
-    """
-    Formats a list ['bla',0,'bla'] into a IMAS path 'bla[0].bla'
-
-    :param path: ODS path format
-
-    :return: IMAS path format
-    """
-    ipath = ''
-    for kstep, step in enumerate(path):
-        if isinstance(step, int) or step == ':':
-            ipath += "[%s]" % step
-        elif kstep == 0:
-            ipath += '%s' % step
-        else:
-            ipath += '.%s' % step
-    return ipath
-
-
-def l2u(path):
-    """
-    Formats a list ['bla',0,'bla'] into a universal path 'bla.:.bla'
-    NOTE: a universal ODS path substitutes lists indices with :
-
-    :param path: list of strings and integers
-
-    :return: universal ODS path format
-    """
-    return o2u(l2o(path))
+    pyximport.install(language_level=3)
+    from .omas_cython import *
+except Exception as _excp:
+    warnings.warn('omas cython failed: ' + str(_excp))
+    with open(os.path.split(__file__)[0] + os.sep + 'omas_cython.pyx', 'r') as f:
+        exec(f.read(), globals())
 
 
 def l2ut(path):
@@ -959,142 +887,37 @@ def l2ut(path):
     return l2o(lpath)
 
 
-def l2o(path):
-    """
-    Formats a list ['bla',0,'bla'] into an ODS path format 'bla.0.bla'
-
-    :param path: list of strings and integers
-
-    :return: ODS path format
-    """
-    return '.'.join(filter(None, map(str, path)))
-
-
-_o2u_pattern = re.compile(r'\.[0-9:]+')
-_o2u_pattern_no_split = re.compile(r'^[0-9:]+')
-_o2i_pattern = re.compile(r'\.([:0-9]+)')
-
-
-def o2u(path):
-    """
-    Converts an ODS path 'bla.0.bla' into a universal path 'bla.:.bla'
-
-    :param path: ODS path format
-
-    :return: universal ODS path format
-    """
-    if '.' in path:
-        return re.sub(_o2u_pattern, '.:', path)
-    else:
-        return re.sub(_o2u_pattern_no_split, ':', path)
-
-
-def i2o(path):
-    """
-    Formats a IMAS path 'bla[0].bla' into an ODS path 'bla.0.bla'
-
-    :param path: IMAS path format
-
-    :return: ODS path format
-    """
-    return path.replace(']', '').replace('[', '.')
-
-
-def o2i(path):
-    """
-    Formats a ODS path 'bla.0.bla' into an IMAS path 'bla[0].bla'
-
-    :param path: ODS path format
-
-    :return: IMAS path format
-    """
-    return re.sub(_o2i_pattern, r'[\1]', path)
-
-
-def u2o(upath, path):
-    """
-    Replaces `:` and integers in `upath` with ':' and integers from in `path`
-    e.g. uo2('a.:.b.:.c.1.d.1.e','f.:.g.1.h.1.i.:.k')) becomes 'bla.1.hello.2.bla'
-
-    :param upath: universal ODS path
-
-    :param path: ODS path
-
-    :return: filled in ODS path
-    """
-    if upath.startswith('1...'):
-        return upath
-    ul = p2l(upath)
-    ol = p2l(path)
-    for k in range(min([len(ul), len(ol)])):
-        if (ul[k] == ':' or isinstance(ul[k], int)) and (ol[k] == ':' or isinstance(ol[k], int)):
-            ul[k] = ol[k]
-        elif ul[k] == ol[k]:
-            continue
-        else:
-            break
-    return l2o(ul)
-
-
-def u2n(upath, list_of_n):
-    """
-    Replaces `:` and integers in `upath` with integers provided
-    e.g. uo2('a.:.b.:.c.1.d.1.e',[2,3]) becomes 'a.2.b.3.c.1.d.1.e'
-
-    :param upath: universal ODS path
-
-    :param list_of_n: list of numbers
-
-    :return: filled in ODS path
-    """
-    ul = p2l(upath)
-    i = 0
-    for k in range(len(ul)):
-        if ul[k] == ':' or isinstance(ul[k], int):
-            if ul[k] == ':' and i < len(list_of_n):
-                ul[k] = list_of_n[i]
-            i += 1
-    return l2o(ul)
-
-
-def trim_common_path(p1, p2):
-    """
-    return paths in lists format trimmed of the common first path between paths p1 and p2
-
-    :param p1: ODS path
-
-    :param p2: ODS path
-
-    :return: paths in list format trimmed of common part
-    """
-    p1 = p2l(p1)
-    p2 = p2l(p2)
-    both = [x if x[0] == x[1] else None for x in zip(p1, p2)] + [None]
-    return p1[both.index(None) :], p2[both.index(None) :]
-
-
-def omas_info(structures=None, imas_version=omas_rcparams['default_imas_version']):
+def omas_info(structures=None, hide_obsolescent=True, cumulative_queries=False, imas_version=omas_rcparams['default_imas_version']):
     """
     This function returns an ods with the leaf nodes filled with their property informations
 
+    :param hide_obsolescent: hide obsolescent entries
+
     :param structures: list with ids names or string with ids name of which to retrieve the info
                        if None, then all structures are returned
 
-    :return: ods
+    :param cumulative_queries: return all IDSs that have been queried
+
+    :param imas_version: IMAS version to look up
+
+    :return: ods showcasing IDS structure
     """
 
+    from omas import ODS
+
     if not structures:
         structures = sorted(list(structures_filenames(imas_version).keys()))
     elif isinstance(structures, str):
         structures = [structures]
 
-    # caching
-    if imas_version not in _info_structures:
-        from omas import ODS
+    # caching based on imas version and obsolescence
+    if (imas_version, hide_obsolescent) not in _info_structures:
+        _info_structures[imas_version, hide_obsolescent] = ODS(imas_version=imas_version, consistency_check=False)
+    ods = _info_structures[imas_version, hide_obsolescent]
 
-        _info_structures[imas_version] = ODS(imas_version=imas_version, consistency_check=False)
-    ods = _info_structures[imas_version]
+    ods_out = ODS(imas_version=imas_version, consistency_check=False)
 
+    # generate ODS with info
     for structure in structures:
         if structure not in ods:
             tmp = load_structure(structure, imas_version)[0]
@@ -1109,9 +932,18 @@ def omas_info(structures=None, imas_version=omas_rcparams['default_imas_version'
                         break
                 if parent:
                     continue
+                if hide_obsolescent and omas_info_node(item).get('lifecycle_status', '') == 'obsolescent':
+                    continue
                 ods[item.replace(':', '0')] = tmp[item]
+        ods_out[structure] = ods[structure]
 
-    return copy.deepcopy(ods)
+    # cumulative queries
+    if cumulative_queries:
+        for structure in ods:
+            if structure not in ods_out:
+                ods_out[structure] = ods[structure]
+
+    return ods_out
 
 
 def omas_info_node(key, imas_version=omas_rcparams['default_imas_version']):
