@@ -11,7 +11,6 @@ Test script for omas/omas_physics.py
 -------
 """
 
-import unittest
 import os
 import numpy
 import warnings
@@ -32,11 +31,24 @@ except ImportError as _excp:
     failed_PINT = _excp
 
 
-class TestOmasPhysics(unittest.TestCase):
+class TestOmasPhysics(UnittestCaseOmas):
     """
     Test suite for omas_physics.py
     """
 
+    def test_check_iter_scenario_requirements(self):
+        from omas.omas_imas import iter_scenario_requirements
+
+        ods = ODS()
+        tmp = ods.physics_check_iter_scenario_requirements()
+        assert tmp == iter_scenario_requirements
+
+        ods.sample_equilibrium()
+        ods.sample_core_profiles()
+        ods.physics_summary_consistent_global_quantities()
+        tmp = ods.physics_check_iter_scenario_requirements()
+        assert tmp != iter_scenario_requirements
+
     def test_equilibrium_consistent(self):
         ods = ODS()
         ods.sample_equilibrium()
@@ -56,7 +68,7 @@ class TestOmasPhysics(unittest.TestCase):
         assert not diff
 
         ods2 = ods.physics_core_profiles_pressures(update=False)
-        assert all(['press' in item for item in ods2.flat().keys() if not item.endswith('rho_tor_norm')])
+        assert all('press' in item for item in ods2.flat().keys() if not item.endswith('rho_tor_norm'))
         return
 
     def test_core_profiles_currents(self):
